/**
	$Id: IncludeContentTabFactory.java 5485 2010-11-11 07:57:08Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.section;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyContent;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.util.error.ThrowableUtil;
import net.sf.basedb.util.extensions.InvokationContext;
import net.sf.basedb.util.extensions.xml.PathSetter;
import net.sf.basedb.util.extensions.xml.PathType;

/**
	A section action factory implementation that includes another resource 
	as the content of a section. The other resource can be a JSP
	script, servlet, a static file, or anything else that makes sense.
	<p>
	The current {@link JspContext} is stored as a {@link ServletRequest} attribute
	with key {@link JspContext#ATTRIBUTE_KEY}. If the included page is a JSP page it
	can get the JspContext by calling 
	<code>request.getAttribute(JspContext.ATTRIBUTE_KEY)</code>.
	
	<p>
	Use the {@link #setScript(String)} and {@link #setStylesheet(String)} methods.
	Each call to these methods will add the argument to a set. Use
	the {@link #getScripts()} and {@link #getStylesheets()} methods to gain
	access to the sets, for example, to remove added items.
	<p>
	
	Note that this factory will accept and generate content for all nodes (including
	folder nodes). If only some nodes should be used, the factory should be subclassed
	and override the {@link #prepareContext(InvokationContext)} method.
	
	@author nicklas
	@since 3.2
	@base.modified $Date$
*/
public class IncludeContentSectionFactory 
	extends AbstractJspActionFactory<SectionAction>
{

	private String id;
	private String showClazz;
	private String hideClazz;
	private String style;
	private String contentStyle;
	private String title;
	private String include;
	
	/**
		Create a new factory instance.
	*/
	public IncludeContentSectionFactory()
	{}
	
	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public SectionAction[] getActions(InvokationContext context) 
	{
		Section section = new Section((JspContext)context.getClientContext());
		return new SectionAction[] { section };
	}
	// ------------------------------------

	public void setId(String id) 
	{
		this.id = id;
	}

	public void setShowClazz(String showClazz) 
	{
		this.showClazz = showClazz;
	}

	public void setHideClazz(String hideClazz) 
	{
		this.hideClazz = hideClazz;
	}

	public void setStyle(String style) 
	{
		this.style = style;
	}
	
	public void setContentStyle(String contentStyle) 
	{
		this.contentStyle = contentStyle;
	}

	public void setTitle(String title) 
	{
		this.title = title;
	}

	@PathSetter(pathType=PathType.CONTEXT_RELATIVE)
	public void setInclude(String include)
	{
		this.include = include;
	}

	class Section
		implements SectionAction
	{

		private final JspContext jspContext;
		
		Section(JspContext jspContext)
		{
			this.jspContext = jspContext;
		}
		
		/*
			From the TabAction interface
			--------------------------------
		*/
		@Override
		public String getId() 
		{
			return id;
		}
	
		@Override
		public String getShowClazz() 
		{
			return showClazz;
		}
	
		@Override
		public String getHideClazz() 
		{
			return hideClazz;
		}

		@Override
		public String getStyle() 
		{
			return style;
		}

		@Override
		public String getContentStyle() 
		{
			return contentStyle;
		}

		@Override
		public String getTitle() 
		{
			return title;
		}

		@Override
		public String getContent()
		{
			PageContext pageContext = jspContext.getPageContext();
			String tabContent = null;
			BodyContent sectionBody = null;
			try
			{
				sectionBody = pageContext.pushBody();
				pageContext.getRequest().setAttribute(JspContext.ATTRIBUTE_KEY, jspContext);
				pageContext.include(include);
				tabContent = sectionBody.getString();
			}
			catch (Exception ex)
			{
				tabContent = "<div class=\"stacktrace\">" + 
					HTML.encodeTags(ThrowableUtil.stackTraceToString(ex)) + 
					"</div>";
			}
			finally
			{
				if (sectionBody != null) pageContext.popBody();
			}
			return tabContent;
		}
	}
	
}
