/*
	$Id: Page.java 6837 2015-04-08 13:55:47Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006, 2008 Johan Enell, Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.core.Version;
import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.extensions.ExtensionsControl;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.extensions.skin.ImageRemapper;
import net.sf.basedb.clients.web.extensions.skin.ImageRemapperUtil;
import net.sf.basedb.clients.web.extensions.skin.SkinAction;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.ActionIterator;
import net.sf.basedb.util.extensions.ExtensionsInvoker;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.http.HttpServletRequest;

/**
	This tag should be used together with the {@link Head &lt;base:head&gt;} and
	{@link Body &lt;base:body&gt;} tags on every JSP page in BASE.
	<p>
	<b>Example:</b>
	<pre>
&lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
&lt;base:page type="default" title="Welcome to BASE"&gt;
&lt;base:head&gt;
   &lt;!-- scripts and other head things... --&gt;
&lt;/base:head&gt;
&lt;base:body&gt;
   Page contents go here....
&lt;/base:body&gt;
&lt;/base:page&gt;
</pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:page
      type=default|popup|include
	  title=...
	  menu=...
	  doctype=...
   &gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>type</td>
		<td>default</td>
		<td>no</td>
		<td>The type of the page. Three values are defined:
			<ul>
			<li>default: The default if no other value is specified.
			<li>popup: Use this value for a popup dialogue.
			<li>include: Use this for pages that are included as part of another page
				(skips generation of &lt;html&gt;, &lt;body&gt;, etc. tags).
			</ul>
		</td>
	</tr>
	<tr>
		<td>title</td>
		<td>-</td>
		<td>yes</td>
		<td>The title of the page.</td>
	</tr>
	<tr>
		<td>menu</td>
		<td>auto</td>
		<td>no</td>
		<td>
			The name of the menu to use on the page. This value is only used
			for pages with type=default. See the <code>/include/menu.jsp</code>
			file for defined menu names.
		</td>
	</tr>
	<tr>
		<td>doctype</td>
		<td>html</td>
		<td>no</td>
		<td>
			The doctype to use in the generated html page. The default is <i>html</i> which
			is the new HTML5 doctype. Use <i>HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"</i>
			to get the older HTML 4 doctype.
		</td>
	</tr>
	<tr>
		<td>favicon</td>
		<td>favicon.ico</td>
		<td>no</td>
		<td>
			The 'favicon' to use.
		</td>
	</tr>
	</table>
	
	@author Nicklas
	@version 2.0
*/
public class Page
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = -5693587214280114326L;

	/**
		This is the default page type.
	*/
	static final int PAGE_TYPE_DEFAULT = 0;
	
	/**
		The page is a popup dialouge.
	*/
	static final int PAGE_TYPE_POPUP = 1;
		
	/**
		This page is an included page.
	*/
	static final int PAGE_TYPE_INCLUDE = 3;
	
	/**
		The page is in a separate iframe .
		@since 3.1
	 */
	static final int PAGE_TYPE_IFRAME = 4;

	/**
		The title of the page.
	*/
	private String title = null;
	
	/**
		The name of the menu that should be displayed on the current page.
		This setting is only used if the type is PAGE_TYPE_DEFAULT
	*/
	private String menu = "auto";

	/**
		Set to TRUE to not use skin on the page.
		@since 3.4
	*/
	private boolean noSkin = false;
	
	/**
		The <!doctype xxx> setting.
	*/
	private String doctype = "html";
	
	/**
		The type of the current page.
	*/
	private int type = PAGE_TYPE_DEFAULT;

	/**
		The 'favicon' to use.
	*/
	private String favicon = null;

	private static volatile boolean initialized = false;
	
	/**
		The name of this webserver.
	*/
	private static String SERVER_NAME = null;

	/**
		The path to the root directory of this webserver.
	*/
	private static String ROOT = null;

	/**
		The version of BASE.
	*/
	private static String BASE_VERSION = null;
	
	private static String MAX_URL_LENGTH = null;

	/**
		The SessionControl object in use for this request.
	*/
	private transient SessionControl sc;
	
	private transient JspContext skinContext;
	
	private transient List<SkinAction> skinActions;
	
	/**
		Static version of {@link #getRoot()}.
		@since 3.5
	*/
	public static String ROOT()
	{
		return ROOT;
	}
	
	/**
		Initialize static fields.
		@since 2.16
	*/
	private static synchronized void initStaticFields(PageContext pageContext)
	{
		if (initialized) return;

		int maxUrlLength = Values.getInt(pageContext.getServletContext().getInitParameter("max-url-length"));
		MAX_URL_LENGTH = maxUrlLength > 0 ? String.valueOf(maxUrlLength) : null;
		ROOT = ((HttpServletRequest)pageContext.getRequest()).getContextPath()+"/";
		BASE_VERSION = "BASE " + Version.getMajor() + "." + 
			Version.getMinor() + "." + Version.getMaintenance() + Version.getSuffix();
		SERVER_NAME = Application.getHostName();
		initialized = true;
	}

	/*
		Taglib initialization methods
	*/
	public void setMenu(String menu)
	{
		this.menu = menu;
	}
	public String getMenu()
	{
		return menu;
	}
	
	public void setTitle(String title)
	{
		this.title = title;
	}
	public String getTitle()
	{
		return title;
	}
	
	/**
		Set to TRUE to disable skins.
		@since 3.4
	*/
	public void setNoskin(boolean noSkin)
	{
		this.noSkin = noSkin;
	}

	public boolean getNoSkin()
	{
		return noSkin;
	}
	
	public void setType(String type)
	{
		if ("popup".equalsIgnoreCase(type))
		{
			this.type = PAGE_TYPE_POPUP;
		}
		else if ("include".equalsIgnoreCase(type))
		{
			this.type = PAGE_TYPE_INCLUDE;
		}
		else if ("iframe".equalsIgnoreCase(type))
		{
			this.type = PAGE_TYPE_IFRAME;
		}
		else
		{
			this.type = PAGE_TYPE_DEFAULT;
		}
	}
	public int getTypeCode()
	{
		return type;
	}
	
	public void setDoctype(String doctype)
	{
		this.doctype = doctype;
	}
	
	public String getDoctype()
	{
		return doctype;
	}
	/**
	
		@since 3.3.1
	*/
	public void setFavicon(String favicon)
	{
		this.favicon = favicon;
	}
	
	public String getFavicon()
	{
		return favicon;
	}
	
	/*
		Server methods
	*/
	public String getRoot()
	{
		return ROOT;
	}
	/**
		@since 2.8
	*/
	public String getMaxUrlLength()
	{
		return MAX_URL_LENGTH;
	}
	public String getServerName()
	{
		return SERVER_NAME;
	}
	public String getBaseVersion()
	{
		return BASE_VERSION;
	}
	public SessionControl getSessionControl()
	{
		return sc;
	}

	/**
		Get the JSP context used for invoking skin extensions. Needed
		by HEAD taglib so that it can include required scripts and stylesheets.
		@since 3.4
	*/
	public JspContext getSkinContext()
	{
		return noSkin ? null : skinContext;
	}
	
	/**
		Get all skin extension actions. Needed in BODY so that
		it can include data on the page.
		@since 3.4
	*/
	public List<SkinAction> getSkinActions()
	{
		return noSkin ? null : skinActions;
	}
	
	@Override
	public void setPageContext(PageContext pageContext)
	{
		super.setPageContext(pageContext);
		favicon = null;
		if (!initialized) initStaticFields(pageContext);
		try
		{
			sc = Base.getSessionControl(pageContext, false);
		}
		catch (RuntimeException ex)
		{}
	}

	@SuppressWarnings("unchecked")
	@Override
	public int doStartTag()
		throws JspException
	{
		if (type != PAGE_TYPE_INCLUDE)
		{
			if (!noSkin)
			{
				// Load skin extensions: net.sf.basedb.clients.web.global-skin
				skinContext = ExtensionsControl.createContext(sc, pageContext);
				skinContext.setAttribute("page-type", getTypeCode());
				ExtensionsInvoker<SkinAction> invoker = (ExtensionsInvoker<SkinAction>)ExtensionsControl.useExtensions(skinContext, "net.sf.basedb.clients.web.global-skin");
				ActionIterator<SkinAction> it = invoker.iterate();
				skinActions = new ArrayList<SkinAction>();
				ImageRemapper remapper = ImageRemapperUtil.getNewImageRemapperIfNeeded(pageContext.getServletContext());
				while (it.hasNext())
				{
					SkinAction skin = it.next();
					if (remapper != null) skin.remapImages(remapper);
					skinActions.add(skin);
					if (favicon == null)
					{
						String skinIcon = skin.getFavicon();
						if (skinIcon != null) favicon = skinIcon;
					}
				}
				if (remapper != null) ImageRemapperUtil.setCurrentMapper(remapper);
			}
			
			StringBuilder sb = new StringBuilder();
			sb.append("<!doctype ").append(getDoctype()).append(">\n");
			sb.append("<html");
			if (getId() != null) sb.append(" id=\"").append(getId()).append("\"");
			sb.append(">\n");
			try
			{
				pageContext.getOut().clearBuffer();
				pageContext.getOut().print(sb.toString());
			}
			catch (Exception ex)
			{
				throw new JspTagException(ex.getMessage());
			}
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag() 
		throws JspException
	{
		if (type != PAGE_TYPE_INCLUDE)
		{
			StringBuilder sb = new StringBuilder();
			sb.append("</html>\n");
			try
			{
				pageContext.getOut().print(sb.toString());
			}
			catch (Exception ex)
			{
				throw new JspTagException(ex.getMessage());
			}
		}
		return EVAL_PAGE;
	}
}



