/*
	$Id: Scripts.java 5426 2010-09-24 09:18:50Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.table;

import java.io.IOException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import net.sf.basedb.clients.web.extensions.list.ListColumnAction;
import net.sf.basedb.core.DbControl;

/**
	Tag for generating content from extension columns.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;tbl:xt-cells
      dc=... 
      item=...
      &gt;
      &lt;tbl:cell column="xt-columns" /&gt;
   &lt;/tbl:xt-cells&gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>dc</td>
		<td>-</td>
		<td>yes</td>
		<td>
			A {@link DbControl} that can be used to query the database.
		</td>
	</tr>
	<tr>
		<td>item</td>
		<td>-</td>
		<td>yes</td>
		<td>
			The current item in the table.
		</td>
	</tr>
	</table>

	@author Nicklas
	@since 3.2
*/
public class ExtensionCells 
	extends BodyTagSupport
{

	private static final long serialVersionUID = -6033350639081278510L;

	private Object item;
	private DbControl dc;
	
	public ExtensionCells()
	{}
	
	
	public void setDc(DbControl dc)
	{
		this.dc = dc;
	}
	
	public void setItem(Object item)
	{
		this.item = item;
	}
	
	private Iterator<ListColumnAction> extensionColumns;
	private ListColumnAction current;
	
	@Override
	public int doStartTag() 
		throws JspException
	{
		// Get all extension columns from the main table...
		Table table = (Table)findAncestorWithClass(this, Table.class);
		List<ListColumnAction> columns = table.getExtensionColumns();
		if (columns == null) return SKIP_BODY;

		// ...and setup iterator for looping through all columns
		extensionColumns = columns.iterator();
		if (!extensionColumns.hasNext()) return SKIP_BODY;
		return EVAL_BODY_INCLUDE;
	}
	
	
	@Override
	public int doAfterBody() 
		throws JspException 
	{
		// Loop until the last extension has been processed
		return extensionColumns.hasNext() ? EVAL_BODY_AGAIN : SKIP_BODY;
	}


	// This class is not serializable
	private void readObject(ObjectInputStream ois)
		throws ClassNotFoundException, IOException 
	{
		throw new NotSerializableException();
	}
	private void writeObject(ObjectOutputStream ois)
		throws IOException 
	{
		throw new NotSerializableException();
	}
	
	/**
		This is called by the Cell taglib when the next extension column
		is about to be processed. The method should step to the next column
		and return the column id.
	*/
	String getNextColumnId()
	{
		current = extensionColumns.next();
		return current.getId();
	}
	
	/**
		Get the value for the current column and item.
	*/
	@SuppressWarnings("unchecked")
	Object getCurrentValue()
	{
		return current.getValue(dc, item);
	}
	
	
}


