/*
	$Id: PositionBatcher.java 6469 2014-06-10 13:52:45Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.ReporterData;
import net.sf.basedb.core.hibernate.ExecuteUpdateWork;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Selects;

import java.util.List;
import java.util.LinkedList;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

/**
	Batcher class for per-position data in the dynamic database. This batcher
	is used to insert data that is valid for all positions in a data cube.
	<p>
	Normally this means linking a position to a reporter.
	<p>
	Code example: {@link <a href="../../../../../development/overview/dynamic/rootbioassayset.html">Dynamic API - Creating a root bioassayset</a>}

	@author Nicklas
	@version 2.0
	@see BioAssaySet#getPositionBatcher()
	@base.modified $Date: 2014-06-10 15:52:45 +0200 (ti, 10 jun 2014) $
*/
public class PositionBatcher
	extends AbstractBatcher
{

	/**
		The datacube this batcher inserts data for.
	*/
	private final DataCube dataCube;
	
	/**
		The number of queued inserts. Used to check when a flush
		is needed.
	*/
	private int queuedInsertCount = 0;
	
	/**
		The total number of inserts done by this batcher.
	*/
	private int totalInsertCount = 0;
	
	/**
		The maximum position number inserted by the batcher.
	*/
	private int maxPosition = 0;
	
	/**
		The number of bytes this batcher has added to the data cube and
		experiment.
	*/
	private long bytes = 0;

	/**
		The number of bytes used by a single row.
	*/
	private final long bytesPerRow;
	
	/**
		The statement that inserts the position/reporter link.
<pre class="code">
INSERT INTO Dynamic#PerPosition (cube, position, reporter_id)
VALUES (cube, ?, ?)
</pre>
	*/
	private PreparedStatement positionSql = null;
	
	/**
	 	The SQL string for the {@link #positionSql} statement.
	*/
	private String positionSqlStatement;

	/**
		Create a new position batcher for a data cube. It is assumed that
		the calling code has checked that the cube is allowed
		to receive more data (ie. it hasn't been committed to the database).
	*/
	PositionBatcher(DbControl dc, DataCube dataCube)
	{
		super();
		this.dataCube = dataCube;
		this.bytesPerRow = VirtualTable.POSITION.getBytesPerRow(dataCube.getVirtualDb());
		setDbControl(dc);
	}

	/*
		From the Batcher interface
		-------------------------------------------
	*/
	@Override
	public void flush()
		throws BaseException
	{
		try
		{
			if (positionSql != null) positionSql.executeBatch();
		}
		catch (SQLException ex)
		{
			logSql.error(positionSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		queuedInsertCount = 0;
	}
	// -------------------------------------------
	/*
		From the AbstractBatcher class
		-------------------------------------------
	*/
	/**
		Update bytes and max position on the datacube and close
		open SQL statements.
	*/
	@Override
	void onBeforeClose()
		throws BaseException
	{
		dataCube.setNumPositions(maxPosition);
		dataCube.addBytes(bytes);
		try
		{
			if (positionSql != null) 
			{
				positionSql.close();
				positionSql = null;
			}
		}
		catch (SQLException ex)
		{
			logSql.error(ex.getMessage(), ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
		}
		if (totalInsertCount > 0 && !getDbControl().isRolledBack())
		{
			String catalog = Application.getDynamicCatalog();
			String schema = Application.getDynamicSchema();
			String table = VirtualTable.POSITION.getTableName(dataCube.getVirtualDb());
			analyzeTable(catalog, schema, table);
		}
	}
	// -------------------------------------------

	/**
		Link a position to a reporter.
		@param position The position coordinate
		@param reporter The reporter or null if no reporter is known
		@throws BaseException If there is an error
	*/
	public void insert(int position, ReporterData reporter)
		throws BaseException
	{
		if (isClosed()) throw new ConnectionClosedException();
		updateLastAccess();
		try
		{
			if (positionSql == null) buildPositionSql();
			positionSql.setInt(1, position);
			if (reporter == null)
			{
				positionSql.setNull(2, Types.INTEGER);
			}
			else
			{
				positionSql.setInt(2, reporter.getId());
			}
			positionSql.addBatch();
			if (maxPosition < position) maxPosition = position;
			bytes += bytesPerRow;
			totalInsertCount++;
		}
		catch (SQLException ex)
		{
			logSql.error(positionSqlStatement, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		if (++queuedInsertCount == getBatchSize())
		{
			flush();
		}
	}
	
	/**
		Insert position/reporter links using a query. The query MUST select data in
		the following order: position, reporterId
		<p>
		Internally this method converts the query into a 
		<code>INSERT INTO ... SELECT ...</code> query, making it very fast,
		since the data for each position doesn't have to be retreived, processed and
		then inserted one row at a time. It is possible to call
		this method multiple times with the same or different query. 
		
		@param query The query that selects the data to be inserted
		@return int The number of rows inserted
		@throws BaseException If there is an error
	*/
	public int insert(final AbstractSqlQuery query)
		throws BaseException
	{
		if (isClosed()) throw new ConnectionClosedException();
		updateLastAccess();
		if (query == null) throw new InvalidUseOfNullException("query");

		// Add static value cubeNo to the selection
		query.resetTemporary();
		query.temporarySelect(
			Selects.expression(
				Expressions.integer(dataCube.getCubeNo()), 
				VirtualColumn.CUBE.getName()
			)
		);

		String selectSql = query.getMainQuery(getDbControl(), true);
		
		// Holds the names of the parameters in the query in the order they appear
		final List<String> parameterOrder = new LinkedList<String>();
		
		// Parse out named parameters
		selectSql = query.parseParameters(selectSql, parameterOrder);
		String insertSql = buildInsertSelectSql(selectSql);

		int rowsInserted = 0;
		try
		{
			rowsInserted = HibernateUtil.doReturningWork(getDbControl().getHibernateSession(), 
				new ExecuteUpdateWork(getSessionControl(), insertSql)
				{
					@Override
					public void setParameters(PreparedStatement ps)
						throws SQLException
					{
						query.setParameters(ps, parameterOrder, 1);
					}
				}
			);
			bytes += rowsInserted * bytesPerRow;
			totalInsertCount += rowsInserted;
		}
		catch (SQLException ex)
		{
			logSql.error(insertSql, ex);
			LogUtil.logSQLExceptionChain(logSql, ex);
			throw new BaseException(ex);
		}
		return rowsInserted;
	}
	
	/**
		Builds the insert SQL statement. Also creates the Dynamic#PerPosition table
		if it doesn't exists.
		@see BatchUtil#buildInsertSql(VirtualDb, VirtualTable, VirtualColumn[], Object[])
	*/
	private void buildPositionSql()
		throws SQLException, BaseException
	{
		VirtualDb vdb = dataCube.getVirtualDb();
		if (!vdb.hasTable(VirtualTable.POSITION))
		{
			vdb.createTables(VirtualTable.POSITION);
		}
		short cube = dataCube.getCubeNo();
		positionSqlStatement = BatchUtil.buildInsertSql(vdb,
			VirtualTable.POSITION,
			VirtualTable.POSITION.getColumns(vdb), 
			new Object[] {cube}
		);
		positionSql = HibernateUtil.getConnection(getDbControl().getHibernateSession()).prepareStatement(positionSqlStatement);
	}
	
	/**
		Build the INSERT INTO ... SELECT ... statment. Also creates the 
		Dynamic#PerPosition table if it doesn't exists.
		@param selectSql The SELECT part of the query
		@see BatchUtil#buildInsertSelectSql(VirtualDb, VirtualTable, VirtualColumn[], String)
	*/
	private String buildInsertSelectSql(String selectSql)
		throws BaseException
	{
		VirtualDb vdb = dataCube.getVirtualDb();
		if (!vdb.hasTable(VirtualTable.POSITION))
		{
			vdb.createTables(VirtualTable.POSITION);
		}
		String sql = BatchUtil.buildInsertSelectSql(vdb, VirtualTable.POSITION, VirtualTable.POSITION.getColumns(vdb), selectSql);
		return sql;
	}
}
