/**
	$Id $

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.authentication;

import net.sf.basedb.core.AuthenticationContext;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.Config;
import net.sf.basedb.util.ClassUtil;
import net.sf.basedb.util.JarClassLoader;
import net.sf.basedb.util.extensions.ActionFactory;
import net.sf.basedb.util.extensions.InvokationContext;
/**
	Wrapper authentication manager factory for backwards compatibility with
	the old authentication system. The implementation checks if 'auth.driver'
	is specified in 'base.config' and forwards all authentication calls
	to the specified driver. If no driver is configured, this manager is 
	automatically disabled (by returning false from the {@link #prepareContext(InvokationContext)}
	method).
	
	@author Nicklas
	@since 3.3
*/
@SuppressWarnings("deprecation")
public class OldAuthenticationWrapperFactory 
	implements ActionFactory<AuthenticationManager>
{
	
	private Authenticator auth;
	private AuthenticationMethod authMethod;
	
	public OldAuthenticationWrapperFactory()
	{
		String driver = Config.getString("auth.driver");
		if (driver != null)
		{
			auth = getAuthenticator(driver);
			authMethod = AuthenticationMethod.getInstance(driver);
		}
	}

	@Override
	public boolean prepareContext(InvokationContext<? super AuthenticationManager> context) 
	{
		return auth != null;
	}

	@Override
	public AuthenticationManager[] getActions(InvokationContext<? super AuthenticationManager> context) 
	{
		AuthenticationContext authContext = (AuthenticationContext)context.getClientContext();
		LoginRequest request = authContext.getLoginRequest();
		AuthenticationManager authManager = new OldAuthenticationManager(request);

		return new AuthenticationManager[] { authManager };
	}
	
	
	private Authenticator getAuthenticator(String driver) 
	{
		Authenticator a;
		try
		{
			String jarPath = net.sf.basedb.util.Values.getStringOrNull(Config.getString("auth.jarpath"));
			ClassLoader loader = jarPath == null ? null : JarClassLoader.getInstance(jarPath);
			a = (Authenticator)ClassUtil.checkAndLoadClass(loader, 
				driver, true, Authenticator.class).newInstance();
			a.init(Config.getString("auth.init"));
		}
		catch (Throwable t)
		{
			throw new BaseException("The authentication driver '"+driver+
				"' could not be loaded. The error message is: "+t.getMessage());
		}
		return a;
	}
	
	class OldAuthenticationManager
		implements AuthenticationManager
	{
		private final LoginRequest request;

		OldAuthenticationManager(LoginRequest request) 
		{
			this.request = request;
		}

		@Override
		public AuthenticatedUser authenticate() 
		{
			AuthenticationInformation info = auth.authenticate(request.getLogin(), request.getPassword());
			
			AuthenticatedUser authUser = new AuthenticatedUser(authMethod, info.login, info.id);
			if (auth.supportsExtraInformation() && info.hasExtraInfo)
			{
				ExtraUserInformation extra = new ExtraUserInformation();
				extra.setAddress(info.address);
				extra.setDescription(info.description);
				extra.setEmail(info.email);
				extra.setFax(info.fax);
				extra.setName(info.name);
				extra.setOrganisation(info.organisation);
				extra.setPhone(info.phone);
				extra.setUrl(info.url);
			}
			return authUser;
		}

	}
	
}
