/*
  $Id: FileData.java 6468 2014-06-10 11:27:02Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;

/**
	This class holds information about a file.
	
	@author enell
	@version 2.0
	@see net.sf.basedb.core.File
	@see <a href="../../../../../../html/developer/api/data_api.files.html">Developer documentation: Files and directories</a>
	@base.modified $Date: 2014-06-10 13:27:02 +0200 (ti, 10 jun 2014) $
	@hibernate.class table="`Files`" lazy="true"
*/
public class FileData
	extends CommonData
	implements DiskConsumableData, LoggableData, SubtypableData
{
	/*
		From the NameableData interface
		-------------------------------------------
	*/
	/**
		Get the name of the item.
		@return A <code>String</code> with the name of the item
		@hibernate.property type="string"
		@hibernate.column name="`name`" length="255" not-null="true" index="name_idx" unique-key="File_uniquefile"
	*/
	@Override
	public String getName()
	{
		return super.getName();
	}
	// -------------------------------------------
	/*
		From the DiskConsumableData interface
		-------------------------------------------
	*/
	private DiskUsageData diskUsage;
	@Override
	public DiskUsageData getDiskUsage()
	{
		if(diskUsage == null) diskUsage = new DiskUsageData();
		return diskUsage;
	}
	void setDiskUsage(DiskUsageData diskUsage)
	{
		this.diskUsage = diskUsage;
	}
	// -------------------------------------------
	/*
		From the SubtypableData interface
		----------------------------------
	*/
	private ItemSubtypeData subtype;
	@Override
	public ItemSubtypeData getItemSubtype()
	{
	   return subtype;
	}
	@Override
	public void setItemSubtype(ItemSubtypeData subtype)
	{
	   this.subtype = subtype;
	}
	// ------------------------------

	private long size;
	/**
		Get the size in bytes for this <code>FileData</code> object.
		@hibernate.property column="`size`" type="long" not-null="true"
	*/
	public long getSize()
	{
		return size;
	}
	public void setSize(long size)
	{
		this.size = size;
	}
	
	private String md5;
	/**
		Get the MD5 hash of the file contents. It is always returned as a string
		with 32 hexadecimal characters.
		@hibernate.property column="`md5`" type="string" length="32" not-null="false"
	*/
	public String getMd5()
	{
		return md5;
	}
	public void setMd5(String md5)
	{
		this.md5 = md5;
	}

	private int action;
	/**
		Get the value of the action variable for this <code>FileData</code>
		object.
		@hibernate.property column="`action`" type="int" not-null="true"
	*/
	public int getAction()
	{
		return action;
	}
	public void setAction(int action)
	{
		this.action = action;
	}
	
	private int location;
	/**
		Get the {@link net.sf.basedb.core.Location} of this <code>FileData</code>
		@hibernate.property column="`location`" type="int" not-null="true"
	*/
	public int getLocation()
	{
		return location;
	}
	public void setLocation(int location)
	{
		this.location = location;
	}
	
	private String mimeType;
	public static final int MAX_MIMETYPE_LENGTH = 255;
	/**
		Get the mime type of this <code>FileData</code> item.
	 	@hibernate.property column="`mimetype`" type="string" length="255"
	*/
	public String getMimeType()
	{
		return mimeType;
	}
	public void setMimeType(String mimeType)
	{
		this.mimeType = mimeType;
	}
	
	private String charset;
	public static final int MAX_CHARSET_LENGTH = 255;
	/**
		Get the character set used in this file (for text-files only).
	 	@hibernate.property column="`charset`" type="string" length="255"
	 	@since 2.9
	*/
	public String getCharacterSet()
	{
		return charset;
	}
	/**
		@since 2.9
	*/
	public void setCharacterSet(String charset)
	{
		this.charset = charset;
	}

	private DirectoryData directory;
	/**
		Get the associated {@link DirectoryData} item.
		@hibernate.many-to-one outer-join="false"
		@hibernate.column name="`directory_id`" not-null="true" unique-key="File_uniquefile"
	*/
	public DirectoryData getDirectory()
	{
		return directory;
	}
	public void setDirectory(DirectoryData directory)
	{
		this.directory = directory;
	}
	
	private boolean writeProtected;
	/**
		If the actual file contents should be write protected or not.
		@since 2.4
		@hibernate.property column="`write_protected`" type="boolean" not-null="true"
	*/
	public boolean isWriteProtected()
	{
		return writeProtected;
	}
	public void setWriteProtected(boolean writeProtected)
	{
		this.writeProtected = writeProtected;
	}

	private Date lastUpdate;
	/**
		Get the date and time the actual file data was last updated.
		@return A <code>Date</code> object, or null if no file has been uploaded
		@since 2.4
		@hibernate.property column="`last_update`" type="timestamp" not-null="false"
	*/
	public Date getLastUpdate()
	{
		return lastUpdate;
	}
	public void setLastUpdate(Date lastUpdate)
	{
		this.lastUpdate = lastUpdate;
	}
	
	private boolean compressed;
	/**
		If the file is stored in a compressed format or not.
		@since 2.5
		@hibernate.property column="`compressed`" type="boolean" not-null="true"
	*/
	public boolean isCompressed()
	{
		return compressed;
	}
	public void setCompressed(boolean compressed)
	{
		this.compressed = compressed;
	}
	
	private long compressedSize;
	/**
		Get the size in bytes that this <code>FileData</code> object
		uses after compression. If the file is not compressed, this value
		is the same as {@link #getSize()}
		@hibernate.property column="`compressed_size`" type="long" not-null="true"
	*/
	public long getCompressedSize()
	{
		return compressedSize;
	}
	public void setCompressedSize(long compressedSize)
	{
		this.compressedSize = compressedSize;
	}
	
	private String internalName;
	/**
		Get the internal name accosiated with this file.
		@hibernate.property column="`internalname'" type="string" length="255" not-null="false" 
	*/
	@NotLoggable
	@PropertyPathProtected
	public String getInternalName()
	{
		return internalName;
	}
	public void setInternalName(String internalName)
	{
		this.internalName = internalName;
	}
	
	private String url;
	public static final int MAX_URL_LENGTH = 2000;
	/**
		Get the URL to the file. This property is only
		set/valid for files that are stored externally (location=EXTERNAL).
		@since 2.16
		@hibernate.property column="`url`" type="text" not-null="false" 
	*/
	public String getUrl()
	{
		return url;
	}
	public void setUrl(String url)
	{
		this.url = url;
	}
	
	private FileServerData fileServer;
	/**
		Get the associated {@link FileServerData} item. This property is only
		set/valid for files that are stored externally (location=EXTERNAL).
		@since 2.16
		@hibernate.many-to-one column="`fileserver_id`" not-null="false" outer-join="false"
	*/
	public FileServerData getFileServer()
	{
		return fileServer;
	}
	public void setFileServer(FileServerData fileServer)
	{
		this.fileServer = fileServer;
	}

	
}
