/*
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.Listable;

/**
	This represents an arbitrary collection of items. While the underlying
	implementation would allow any kind of items to be stored in the list,
	considerations for GUI support limit the items to those that implement
	the {@link Listable} interface.
	
	@author Nicklas
	@since 3.5
	@see net.sf.basedb.core.ItemList
	@hibernate.class table="`ItemLists`" lazy="false"
*/
public class ItemListData
	extends AnnotatedData
{
	public ItemListData()
	{}

	/**
		The maximum length of the external ID that can be stored in the database.
		@see #setExternalId(String)
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = 255;
	private String externalId;
	/**
		Get the external id for this <code>ItemList</code> item.
		@hibernate.property column="`external_id`" type="string" length="255" not-null="false"
	*/
	public String getExternalId()
	{
		return externalId;
	}
	public void setExternalId(String externalId)
	{
		this.externalId = externalId;
	}

	private int memberType;
	/**
		Get the type of items this list can hold. Not possible to change
		after creation.
		@hibernate.property column="`member_type`" type="int" not-null="true" update="false"
	*/
	public int getMemberType()
	{
		return memberType;
	}
	public void setMemberType(int memberType)
	{
		this.memberType = memberType;
	}
	
	private ItemSubtypeData subtype;
	/**
		Get the subtype of items that can be members in this list. This is
		a recommendation only and is not enforced.
		@hibernate.many-to-one column="`subtype_id`" not-null="false" outer-join="false"
	*/
	public ItemSubtypeData getItemSubtype()
	{
		return subtype;
	}
	public void setItemSubtype(ItemSubtypeData subtype)
	{
		this.subtype = subtype;
	}

	private String rawDataType;
	/**
		Get the raw data type of raw bioassays that can be members in this list. 
		This is a recommendation only and is not enforced.
		@hibernate.property column="`rawdatatype`" type="string" length="255" not-null="false"
	*/
	public String getRawDataType()
	{
		return rawDataType;
	}
	public void setRawDataType(String rawDataType)
	{
		this.rawDataType = rawDataType;
	}
	
	private int size;
	/**
		The number of biomaterials in the list.
		@hibernate.property column="`size`" type="int" not-null="true"
	*/
	public int getSize()
	{
		return size;
	}
	public void setSize(int size)
	{
		this.size = size;
	}

	private Set<Integer> members;
	/**
		Get the set that manages which items that are part of this list.
		@hibernate.set table="`ItemListMembers`" lazy="true"
		@hibernate.collection-key column="`list_id`"
		@hibernate.collection-element column="`item_id`" type="int"
	*/
	public Set<Integer> getMembers()
	{
		if (members == null) members = new HashSet<Integer>();
		return members;
	}
	public void setMembers(Set<Integer> members)
	{
		this.members = members;
	}
	
	private boolean disableSyncFilters;
	/**
		If set, syncing members with sync filter is disabled
		@hibernate.property column="`disable_syncfilters`" type="boolean" not-null="true" 
	*/
	public boolean getDisableSyncFilters()
	{
		return disableSyncFilters;
	}
	public void setDisableSyncFilters(boolean disableSyncFilters)
	{
		this.disableSyncFilters = disableSyncFilters;
	}

	private boolean disableManualMembers;
	/**
		If set, manually adding/removing members is disabled
		@hibernate.property column="`disable_manual`" type="boolean" not-null="true" 
	*/
	public boolean getDisableManualMembers()
	{
		return disableManualMembers;
	}
	public void setDisableManualMembers(boolean disableManualMembers)
	{
		this.disableManualMembers = disableManualMembers;
	}
	
	private Date syncDate;
	/**
		Get the date and time the list members was last synchronized 
		by the filter context.
		@hibernate.property column="`sync_date`" type="timestamp" not-null="false"
	*/
	public Date getSyncDate()
	{
		return syncDate;
	}
	public void setSyncDate(Date syncDate)
	{
		this.syncDate = syncDate;
	}
	
	private Set<ItemListSyncFilterData> syncFilters;
	/**
		This is the inverse end.
		@see ItemListSyncFilterData#getItemList()
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`itemlist_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.ItemListSyncFilterData"
	*/
	public Set<ItemListSyncFilterData> getSyncFilters()
	{
		if (syncFilters == null) syncFilters = new HashSet<ItemListSyncFilterData>();
		return syncFilters;
	}
	
	void setSyncFilters(Set<ItemListSyncFilterData> syncFilters)
	{
		this.syncFilters = syncFilters;
	}

	
}
