/**
	$Id: DerivedBioAssayLoader.java 6755 2015-02-20 12:00:55Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.loader;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemResultIterator;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;
import net.sf.basedb.util.overview.Fix;
import net.sf.basedb.util.overview.Node;
import net.sf.basedb.util.overview.NodeAttribute;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.OverviewUtil;
import net.sf.basedb.util.overview.Validator;
import net.sf.basedb.util.overview.filter.ItemTypeFilter;
import net.sf.basedb.util.overview.node.ChildNodeDirection;
import net.sf.basedb.util.overview.node.NameableNameGenerator;
import net.sf.basedb.util.overview.node.NodeFactory;

/**

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2015-02-20 13:00:55 +0100 (fr, 20 feb 2015) $
*/
public class DerivedBioAssayLoader
	extends BasicItemNodeLoader<DerivedBioAssay>
{
	public DerivedBioAssayLoader()
	{
		super(Item.DERIVEDBIOASSAY, ALLOW_ROOT_NODE, 
				new NameableNameGenerator<DerivedBioAssay>("bioassayset", "Bioassay"));
	}
	
	/*
		From the NodeLoader interface
		------------------------------
	*/
	@Override
	public Node createRootNode(DbControl dc, OverviewContext context, DerivedBioAssay item)
	{
		Node node = super.createRootNode(dc, context, item);
		Extract extract = getExtract(item);
		if (extract != null) node.setAttribute(NodeAttribute.EXTRACT, extract);
		return node;
	}

	/**
		Create forward-loading bioassay nodes from a given physical bioassay node or a
		parent derived bioassay set node. The returned node is a folder-type node containing 
		item nodes for each bioassay set.
		@return The folder node for the bioassay sets
	*/
	@Override
	public Node createForwardNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node returnNode = null;
		Item parentType = parentNode.getItemType();
		if (parentType == Item.PHYSICALBIOASSAY)
		{
			returnNode = createForwardNode((PhysicalBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentType == Item.DERIVEDBIOASSAY)
		{
			returnNode = createForwardNode((DerivedBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentType == Item.ITEMLIST)
		{
			ItemListLoader.loadMemberNodes(this, dc, context, parentNode);
		}
		return returnNode;
	}
	
	/**
		Create a reverse-loading bioassay node from a cild bioassay node or
		a raw bioassay node.
		@return A bioassay node, or null if the raw bioassay doesn't have a parent bioassay
	*/
	@Override
	public Node createReverseNode(DbControl dc, OverviewContext context, Node parentNode)
	{
		Node returnNode = null;
		Item parentType = parentNode.getItemType();
		if (parentType == Item.DERIVEDBIOASSAY)
		{
			returnNode = createReverseNode((DerivedBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		else if (parentType == Item.RAWBIOASSAY)
		{
			returnNode = createReverseNode((RawBioAssay)parentNode.getItem(dc), dc, context, parentNode);
		}
		return returnNode;
	}
	// --------------------------------
	/*
		From the AbstractNodeLoader class
		----------------------------------
	*/
	/**
		Loads property nodes of a derived bioassay set. 
		<ul>
		<li>Annotations: {@link AnnotationLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Data files: {@link DataFileLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Protocol: {@link ProtocolLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Hardware: {@link HardwareLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		<li>Software: {@link SoftwareLoader#createPropertyNode(DbControl, OverviewContext, Node)}
		</ul>
	*/
	@Override
	protected void loadPropertyChildNodes(DbControl dc, OverviewContext context, Node bioAssayNode)
	{
		getNodeLoader(context, Item.EXTRACT).createReverseNode(dc, context, bioAssayNode);
		getNodeLoader(context, Item.ANNOTATION).createPropertyNode(dc, context, bioAssayNode);
		getNodeLoader(context, Item.FILESETMEMBER).createPropertyNode(dc, context, bioAssayNode);
		getNodeLoader(context, Item.PROTOCOL).createPropertyNode(dc, context, bioAssayNode);
		getNodeLoader(context, Item.HARDWARE).createPropertyNode(dc, context, bioAssayNode);
		getNodeLoader(context, Item.SOFTWARE).createPropertyNode(dc, context, bioAssayNode);
		getNodeLoader(context, Item.ANYTOANY).createPropertyNode(dc, context, bioAssayNode);
	}

	/**
		Loads all child derived bioassay and raw bioassay nodes.
		@see DerivedBioAssayLoader#createForwardNode(DbControl, OverviewContext, Node)
		@see RawBioAssayLoader#createForwardNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadForwardChildNodes(DbControl dc, OverviewContext context, Node parentNode)
	{
		getNodeLoader(context, Item.DERIVEDBIOASSAY).createForwardNode(dc, context, parentNode);
		getNodeLoader(context, Item.RAWBIOASSAY).createForwardNode(dc, context, parentNode);
	}
	
	/**
		Loads the parent bioassay or the parent physical bioassay.
		@see PhysicalBioAssayLoader#createReverseNode(DbControl, OverviewContext, Node)
		@see DerivedBioAssayLoader#createReverseNode(DbControl, OverviewContext, Node)
	*/
	@Override
	protected void loadReverseChildNodes(DbControl dc, OverviewContext context, Node bioAssayNode)
	{
		DerivedBioAssay dbas = (DerivedBioAssay)bioAssayNode.getItem(dc);
		if (dbas.isRoot())
		{
			getNodeLoader(context, Item.PHYSICALBIOASSAY).createReverseNode(dc, context, bioAssayNode);
		}
		else
		{
			getNodeLoader(context, Item.DERIVEDBIOASSAY).createReverseNode(dc, context, bioAssayNode);
		}
	}
	// ---------------------------------------

	
	/**
		Create forward-loading derived bioassay nodes that have the given
		physical bioassay as the immediate parent (eg. root bioassays).
	*/
	private Node createForwardNode(PhysicalBioAssay bioAssay, DbControl dc, OverviewContext context, Node bioAssayNode)
	{
		NodeFactory<DerivedBioAssay> nf = getNodeFactory(dc, context);
		Node folderNode = null;
		ItemQuery<DerivedBioAssay> query = context.initQuery(bioAssay.getRootDerivedBioAssays(), "name");
		
		// We should only load derived bioassays that has a null extract or an
		// extract that is found in the parent chain
		Set<Integer> extractIds = getExtractChain(bioAssayNode);
		if (extractIds.size() > 0)
		{
			query.restrict(
				Restrictions.or(
					Restrictions.eq(Hql.property("extract"), null),
					Restrictions.in(Hql.property("extract"), Expressions.parameter("extracts"))
				));
			query.setParameter("extracts", extractIds, Type.INT);
		}

		ItemResultIterator<DerivedBioAssay> it = query.iterate(dc);
		while (it.hasNext())
		{
			DerivedBioAssay child = it.next();
			if (folderNode == null)
			{
				folderNode = new Node("bioassays", "Bioassays", bioAssayNode, ChildNodeDirection.FORWARD);
			}
			Object cacheKey = extractIds.size() > 0 ? null : child;
			createItemNode(nf, child, cacheKey, false, folderNode, ChildNodeDirection.FORWARD);
		}
		postValidateFolder(nf, folderNode, bioAssayNode, false);
		return folderNode;
	}

	/**
		Create forward-loading derived bioassay set nodes that have the given
		derived bioassay set as the parent
	*/
	private Node createForwardNode(DerivedBioAssay parent, DbControl dc, OverviewContext context, Node parentNode)
	{
		NodeFactory<DerivedBioAssay> nf = getNodeFactory(dc, context);
		Node folderNode = null;

		ItemQuery<DerivedBioAssay> query = context.initQuery(parent.getChildren(), "name");
		
		// We should only load derived bioassays that has a null extract or an
		// extract that is found in the parent chain
		Set<Integer> extractIds = getExtractChain(parentNode);
		if (extractIds.size() > 0)
		{
			query.restrict(
				Restrictions.or(
					Restrictions.eq(Hql.property("extract"), null),
					Restrictions.in(Hql.property("extract"), Expressions.parameter("extracts"))
				));
			query.setParameter("extracts", extractIds, Type.INT);
		}

		ItemResultIterator<DerivedBioAssay> it = query.iterate(dc);
		while (it.hasNext())
		{
			DerivedBioAssay child = it.next();
			if (folderNode == null)
			{
				folderNode = new Node("bioassays", "Child bioassays", parentNode, ChildNodeDirection.FORWARD);
			}
			Object cacheKey = extractIds.size() > 0 ? null : child;
			createItemNode(nf, child, cacheKey, false, folderNode, ChildNodeDirection.FORWARD);
		}
		postValidateFolder(nf, folderNode, parentNode, false);
		return folderNode;
	}

	/**
		Create a reverse-loading bioassay node for the bioassay that is the parent
		to the given raw bioassay.
	*/
	private Node createReverseNode(RawBioAssay raw, DbControl dc, OverviewContext context, Node rawNode)
	{
		NodeFactory<DerivedBioAssay> nf = getNodeFactory(dc, context);
		DerivedBioAssay bioAssay = null;
		Extract extract = null;
		boolean denied = false;
		try
		{
			bioAssay = raw.getParentBioAssay();
			// We must set the EXTRACT attribute so that upstream loaders
			// follow the correct path when reaching the biomaterials section
			if (bioAssay != null) extract = getExtract(bioAssay);
		}
		catch (PermissionDeniedException ex)
		{
			denied = true;
		}
		Node bioAssayNode = createItemNode(nf, bioAssay, null, denied, 
				rawNode, ChildNodeDirection.REVERSE);
		if (extract != null && bioAssayNode != null)
		{
			bioAssayNode.setAttribute(NodeAttribute.EXTRACT, extract);
		}
		return bioAssayNode;
	}

	/**
		Create a reverse-loading bioassay node for the bioassay that is
		the parent to the given child bioassay.
	*/
	private Node createReverseNode(DerivedBioAssay child, DbControl dc, OverviewContext context, Node childNode)
	{
		NodeFactory<DerivedBioAssay> nf = getNodeFactory(dc, context);
		
		ItemQuery<DerivedBioAssay> query = context.initQuery(child.getParents(), "name");
		List<DerivedBioAssay> parents = query.list(dc);
		
		Node folderNode = null;
		Node bioAssayNode = null;
		if (parents.size() > 1)
		{
			folderNode = new Node("parents", "Parents", childNode, ChildNodeDirection.REVERSE);
		}
		
		for (DerivedBioAssay dba : parents)
		{
			// We need to handle circular references inline because
			// we can never require that a validator implementation handles this
			Node circularNode = OverviewUtil.getCircularRef(childNode, dba);
			if (circularNode != null)
			{
				context.createFailure(Validator.CIRCULAR_REFERENCE, childNode, 
						"Circular reference to '" + dba.getName() + "'", 
						new Fix("Modify parents of '" + child.getName() + "'", child),
						new Fix("Modify parents of '" + dba.getName() + "'", dba)
					);
			}
			else
			{
				Extract extract = getExtract(dba);
				bioAssayNode = createItemNode(nf, dba, null, false, folderNode == null ? childNode : folderNode, ChildNodeDirection.REVERSE);
				if (extract != null && bioAssayNode != null)
				{
					bioAssayNode.setAttribute(NodeAttribute.EXTRACT, extract);
				}
			}
		}
		postValidateFolder(nf, folderNode, childNode, bioAssayNode == null);
		return folderNode == null ? bioAssayNode : folderNode;
	}

	/**
		Get the extract that is associated with the current derived bioassay
		@return The extract or null if none was found
	*/
	private Extract getExtract(DerivedBioAssay bioAssay)
	{
		Extract extract = null;
		try
		{
			extract = bioAssay.getExtract();
		}
		catch (PermissionDeniedException ex)
		{}
		return extract;
	}
	
	private Set<Integer> getExtractChain(Node node)
	{
		List<Node> extractNodes = node.findAll(new ItemTypeFilter(Item.EXTRACT));
		Set<Integer> extractIds = new HashSet<Integer>(extractNodes.size());
		for (Node n : extractNodes)
		{
			extractIds.add(n.getItem().getId());
		}
		return extractIds;
	}
	
}
