/**
	$Id $

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.query;

import java.util.Iterator;

import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;


/**
	Helper class for executing multiple queries and return the result as 
	a single iterator. The queries are executed in the order that they 
	are given, if a query doesn't return any result, the iterator continues
	with the next query. As soon as a query returns at least one
	result, no more queries are executed.
	
	@author nicklas
	@since 3.2
*/
public class MultiQueryIterator<E extends BasicItem> 
	implements Iterator<E>
{

	/**
		Create a new iterator for the given list of queries. There must
		be at least one query.
		@param dc The DbControl to use when executing the queries
		@param queries An array of queries to execute
	*/
	@SafeVarargs
	public static <E extends BasicItem> Iterator<E> get(DbControl dc, ItemQuery<? extends E>... queries)
	{
		if (queries == null || queries.length == 0)
		{
			throw new NullPointerException("queries");
		}
		return new MultiQueryIterator<E>(dc, queries);
	}
	
	private final DbControl dc;
	private final ItemQuery<? extends E>[] queries;
	
	private Iterator<? extends E> current;
	private int nextQuery;
	private int numReturned;
	
	@SafeVarargs
	private MultiQueryIterator(DbControl dc, ItemQuery<? extends E>... queries)
	{
		this.dc = dc;
		this.queries = queries;
		this.current = queries[0].iterate(dc);
		nextQuery = 1;
	}
	
	/*
		From the Iterator interface
		---------------------------
	*/
	@Override
	public boolean hasNext() 
	{
		while (!current.hasNext() && nextQuery < queries.length)
		{
			if (numReturned > 0) return false;
			if (queries[nextQuery] != null) 
			{
				current = queries[nextQuery].iterate(dc);
			}
			nextQuery++;
		}
		return current.hasNext();
	}

	@Override
	public E next() 
	{
		if (hasNext()) numReturned++;
		return current.next();
	}

	@Override
	public void remove() 
	{
		throw new UnsupportedOperationException();
	}
	// ------------------------------
	
}
