/**
	$Id $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.batchimport;

import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DerivedBioAssay;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Hardware;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.PathParameterType;
import net.sf.basedb.core.PhysicalBioAssay;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.Protocol;
import net.sf.basedb.core.Software;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.util.parser.DefaultItemFunction;
import net.sf.basedb.util.parser.FlatFileParser;
import net.sf.basedb.util.parser.Mapper;

import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
	Plug-in for importing derived bioassay items in a batch. The plug-in can create new 
	items and update existing items.

	@author nicklas
	@since 3.0
	@base.modified $Date $
*/
public class DerivedBioAssayImporter
	extends AbstractItemImporter<DerivedBioAssay>
{

	private static final Set<GuiContext> guiContexts =  
		Collections.singleton(new GuiContext(Item.DERIVEDBIOASSAY, GuiContext.Type.LIST));
	
	/**
		Column mapping parameter for the parent type.
	*/
	protected static final PluginParameter<String> parentTypeColumnMapping = 
		new PluginParameter<String>(
				"parentTypeColumnMapping",
				"Parent type",
				"Mapping that picks the data column specifying the type of parent item. " +
				"Recognised values are PHYSICALBIOASSAY and DERIVEDBIOASSAY. If the parent " +
				"type is not specified, the importer will try to guess a parent type from the " +
				"item subtype. Example: \\Parent type\\\n\n" +
				"NOTE! The parent type can't be changed for existing bioassays.",
				optionalColumnMapping
		);

	/**
		Column mapping parameter for the parent (physical bioassay OR derived bioassay)
	*/
	protected static final PluginParameter<String> parentColumnMapping = 
		new PluginParameter<String>(
				"parentColumnMapping",
				"Parent",
				"Mapping that picks the name or id of the parent item (physical or derived bioassay) " +
				"in the data columns. Example: \\Parent\\\n\n" +
				"NOTE! The importer can only add parents, not replace or remove existing parents.",
				optionalColumnMapping
		);

	protected static final PluginParameter<String> extractColumnMapping = new PluginParameter<String>(
			"extractColumnMapping",
			"Parent extract",
			"Mapping that picks the name or ID of the parent extract from the data columns. " +
			"The plug-in will first try to find an extract with the given name. If none is found and " +
			"the value is numeric it will try to load by internal ID. " +
			"Example: \\Parent extract\\\n\n" +
			"NOTE! This column may not be needed when creating new items if the parent is a " +
			"derived bioassay that already contains a link to an extract (which will be copied)",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> hardwareColumnMapping = 
		new PluginParameter<String>(
			"hardwareColumnMapping",
			"Hardware",
			"Mapping that picks the name or ID of the hardware that was used to create the derived bioassay " +
			"from the data columns. The plug-in will first try to find an item with the given name. If none " +
			"is found and the value is numeric it will try to load by internal ID. " +
			"Example: \\Hardware\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> softwareColumnMapping = 
		new PluginParameter<String>(
			"softwareColumnMapping",
			"Software",
			"Mapping that picks the name or ID of the software that was used to create the derived " +
			"bioassay from the data columns. The plug-in will first try to find an item with the given " +
			"name. If none is found and the value is numeric it will try to load by internal ID. " +
			"Example: \\Software\\",
			optionalColumnMapping
			);

	protected static final PluginParameter<String> fileTypeColumnMapping = 
		new PluginParameter<String>(
				"fileTypeColumnMapping",
				"Data file type",
				"Mapping that picks the name or ID of a data file type from the data columns. " +
				"The plug-in will first try to find a file type with the given name. If none is " +
				"found and the value is numeric it will try to load by internal ID. " +
				"Example: \\File type\\",
				optionalColumnMapping
		);
	
	protected static final PluginParameter<String> fileColumnMapping = 
		new PluginParameter<String>(
				"fileColumnMapping",
				"File",
				"Mapping that picks the path of a data file from the columns. " +
				"The path can be an absolute path (starting with /) or relative " +
				"to the 'Data directory' parameter." +
				"Example: \\File\\",
				optionalColumnMapping
		);

	
	public DerivedBioAssayImporter()
	{}

	/*
		From the InteractivePlugin interface
		------------------------------------
	*/
	@Override
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}
	// ------------------------------------

	/*
		From the AbstractItemImporter class
		------------------------------------
	*/
	
	private Mapper nameMapper;
	private Mapper descriptionMapper;
	private Mapper protocolMapper;
	private DefaultItemFunction<Protocol> defaultProtocolFunction;
	private Mapper hardwareMapper;
	private DefaultItemFunction<Hardware> defaultHardwareFunction;
	private Mapper softwareMapper;
	private DefaultItemFunction<Software> defaultSoftwareFunction;
	private Mapper parentTypeMapper;
	private Mapper parentMapper;
	private Mapper extractMapper;
	private Mapper fileTypeMapper;
	private Mapper fileMapper;
	private String dataDirectory;

	/**
		Useable methods are:
		<ul>
		<li>{@link PropertyIdMethod#NAME}
		<li>{@link InternalIdMethod#INTERNAL_ID}
		<li>{@link FallbackIdMethod#NAME_OR_ID}
		</ul>
	*/
	@Override
	protected IdMethod[] getIdMethods()
	{
		return new IdMethod[] { PropertyIdMethod.NAME,  
				InternalIdMethod.INTERNAL_ID, FallbackIdMethod.NAME_OR_ID};
	}
	
	@Override
	protected Item getItemForSubtypes()
	{
		return Item.DERIVEDBIOASSAY;
	}
	
	/**
		Add parameter for "Data directory" directory where data files may be located. 
	*/
	@Override
	protected void addMoreJobParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(new PluginParameter<String>(
				"dataDirectory",
				"Data directory",
				"A base directory were data files are located. Relative file paths are " +
				"resolved against this directory. If no directory is given, only absolute " +
				"paths (=paths starting with /) can be used.",
				new PathParameterType(Path.Type.DIRECTORY))
			);
	}

	
	/**
		Adds column mappings for name, internalId, subtype, description,
		protocol, hardware, software, parent, data files and permissions.
	*/
	@Override
	protected void addMoreColumnMappingParameters(List<PluginParameter<?>> parameters)
	{
		parameters.add(internalIdColumnMapping);
		parameters.add(nameColumnMapping);
		parameters.add(subtypeColumnMapping);
		parameters.add(descriptionColumnMapping);
		parameters.add(parentTypeColumnMapping);
		parameters.add(parentColumnMapping);
		parameters.add(extractColumnMapping);
		parameters.add(protocolColumnMapping);
		parameters.add(hardwareColumnMapping);
		parameters.add(softwareColumnMapping);
		parameters.add(fileTypeColumnMapping);
		parameters.add(fileColumnMapping);
		parameters.add(permissionTemplateColumnMapping);
	}
	
	/**
		Calls {@link DerivedBioAssay#getQuery()}.
	*/
	@Override
	protected ItemQuery<DerivedBioAssay> createItemQuery()
	{
		return DerivedBioAssay.getQuery();
	}
		
	@Override
	protected void createColumnMappers(FlatFileParser ffp, boolean cropStrings)
	{
		nameMapper = getMapper(ffp, (String)job.getValue("nameColumnMapping"), 
				cropStrings ? DerivedBioAssay.MAX_NAME_LENGTH : null, null);
		descriptionMapper = getMapper(ffp, (String)job.getValue("descriptionColumnMapping"), 
				cropStrings ? DerivedBioAssay.MAX_DESCRIPTION_LENGTH : null, null);
		parentTypeMapper = getMapper(ffp, (String)job.getValue("parentTypeColumnMapping"), null, null);
		parentMapper = getMapper(ffp, (String)job.getValue("parentColumnMapping"), null, null);
		extractMapper = getMapper(ffp, (String)job.getValue("extractColumnMapping"), null, null);
		defaultProtocolFunction = getDefaultItemFunction(Item.PROTOCOL);
		protocolMapper = getMapper(ffp, (String)job.getValue("protocolColumnMapping"), null, null, defaultProtocolFunction);
		defaultHardwareFunction = getDefaultItemFunction(Item.HARDWARE);
		hardwareMapper = getMapper(ffp, (String)job.getValue("hardwareColumnMapping"), null, null, defaultHardwareFunction);
		defaultSoftwareFunction = getDefaultItemFunction(Item.SOFTWARE);
		softwareMapper = getMapper(ffp, (String)job.getValue("softwareColumnMapping"), null, null, defaultSoftwareFunction);
		fileTypeMapper = getMapper(ffp, (String)job.getValue("fileTypeColumnMapping"), null, null);
		fileMapper = getMapper(ffp, (String)job.getValue("fileColumnMapping"), null, null);
		dataDirectory = (String)job.getValue("dataDirectory");
		if (dataDirectory != null && !dataDirectory.endsWith("/"))
		{
			dataDirectory += "/";
		}
	}
	
	@Override
	protected DerivedBioAssay createItem(DbControl dc, FlatFileParser.Data data)
	{
		// We need to know the parent item before we can create a new derived bioassay...
		Item parentType = null;
		if (parentTypeMapper != null)
		{
			// Is the parent type explicitely mappped?
			String type = parentTypeMapper.getValue(data);
			if (type != null) parentType = Item.valueOf(type);
		}
		
		// Get the subtype of the new derived bioassay
		ItemSubtype subtype = findItemSubtype(dc, Item.DERIVEDBIOASSAY, data);
		ItemSubtype parentSubtype = null;		
		if (subtype != null)
		{
			if (parentType != null)
			{
				parentSubtype = subtype.getRelatedSubtype(parentType);
			}
			else
			{
				// Try to find related physical bioassay subtype
				parentSubtype = subtype.getRelatedSubtype(Item.PHYSICALBIOASSAY);
				if (parentSubtype == null)
				{
					// Try to find related derived bioassay type
					parentSubtype = subtype.getRelatedSubtype(Item.DERIVEDBIOASSAY);
				}
			}
		}
		
		if (parentType == null && parentSubtype != null)
		{
			parentType = parentSubtype.getMainItemType();
		}
		boolean isRoot = parentType != Item.DERIVEDBIOASSAY;
		DerivedBioAssay bioAssay = DerivedBioAssay.getNew(dc, isRoot, job.getJob());
				
		if (subtype != null) bioAssay.setItemSubtype(subtype);
		updateItem(dc, bioAssay, data);
		return bioAssay;
	}
	
	@Override
	protected void updateItem(DbControl dc, DerivedBioAssay bioAssay, FlatFileParser.Data data)
	{
		if (nameMapper != null) bioAssay.setName(nameMapper.getValue(data));
		if (descriptionMapper != null) bioAssay.setDescription(descriptionMapper.getValue(data));
		if (bioAssay.isInDatabase()) updateItemSubtype(dc, bioAssay, data);
		if (extractMapper != null) 
		{
			String nameOrId = extractMapper.getValue(data);
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, bioAssay, Item.EXTRACT, 0);
			Extract extract = findExtract(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId, type);
			if (nameOrId == null || extract != null) bioAssay.setExtract(extract);
		}
		if (protocolMapper != null) 
		{
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, bioAssay, Item.PROTOCOL, 0);
			defaultProtocolFunction.setCurrentSubtype(type);
			String nameOrId = protocolMapper.getValue(data);
			Protocol protocol = findProtocol(dc, FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, nameOrId, type);
			if (nameOrId == null || protocol != null) bioAssay.setProtocol(protocol);
		}
		if (hardwareMapper != null) 
		{
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, bioAssay, Item.HARDWARE, 0);
			defaultHardwareFunction.setCurrentSubtype(type);
			String nameOrId = hardwareMapper.getValue(data);
			Hardware hardware = findHardware(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, type);
			if (nameOrId == null || hardware != null) bioAssay.setHardware(hardware);
		}
		if (softwareMapper != null) 
		{
			ItemSubtype type = ItemSubtype.getRelatedSubtype(dc, bioAssay, Item.SOFTWARE, 0);
			defaultSoftwareFunction.setCurrentSubtype(type);
			String nameOrId = softwareMapper.getValue(data);
			Software software = findSoftware(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, type);
			if (nameOrId == null || software != null) bioAssay.setSoftware(software);
		}
		
		updateMultiLineItem(dc, bioAssay, data, 0);
	}
	
	/**
		Add data files
	*/
	@Override
	protected void updateMultiLineItem(DbControl dc, DerivedBioAssay bioAssay, 
		FlatFileParser.Data data, int multiLineNum)
	{
		if (fileTypeMapper != null && fileMapper != null)
		{
			String fileTypeNameOrId = fileTypeMapper.getValue(data);
			String filePath = fileMapper.getValue(data);
			DataFileType fileType = findDataFileType(dc, 
					FallbackIdMethod.NAME_OR_EXTERNALID_OR_ID, fileTypeNameOrId);
			if (fileType != null)
			{
				File file = null;
				if (filePath != null)
				{
					if (!filePath.startsWith("/") && dataDirectory != null) 
					{
						filePath = dataDirectory + filePath;
					}
					file = File.getByPath(dc, new Path(filePath, Path.Type.FILE), false);
				}
				setOrAddFile(bioAssay, fileType, file);
			}
		}
		
		if (parentMapper != null)
		{
			String nameOrId = parentMapper.getValue(data);
			Item parentType = bioAssay.isRoot() ? Item.PHYSICALBIOASSAY : Item.DERIVEDBIOASSAY;
			
			ItemSubtype parentSubtype = ItemSubtype.getRelatedSubtype(dc, bioAssay, parentType);
			if (parentType == Item.DERIVEDBIOASSAY)
			{
				DerivedBioAssay parent = findDerivedBioAssay(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, parentSubtype);
				bioAssay.addParent(parent);
			}
			else
			{
				PhysicalBioAssay pba = findPhysicalBioAssay(dc, FallbackIdMethod.NAME_OR_ID, nameOrId, parentSubtype);
				bioAssay.addPhysicalBioAssay(pba);
			}
		}
	}
	
	/**
		Validate data files if they have been set.
	*/
	@Override
	protected void doneWithItem(DbControl dc, DerivedBioAssay bioAssay, int numMultiLines)
	{
		if (bioAssay.hasFileSet()) bioAssay.getFileSet().validate(dc);
	}
	// ------------------------------------
}
