/**
	$Id: FileUtil.java 6860 2015-04-15 07:15:25Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test;

import java.io.InputStream;

import org.apache.tools.bzip2.CBZip2InputStream;

import com.ice.tar.TarInputStream;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileSetMember;
import net.sf.basedb.core.FileStoreEnabled;
import net.sf.basedb.core.FileStoreUtil;
import net.sf.basedb.core.ItemSubtype;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.SystemItems;

/**

 @author nicklas
 @version 2.2
 @base.modified $Date: 2015-04-15 09:15:25 +0200 (on, 15 apr 2015) $
 */
public class FileUtil
{

	/**
		Upload a file to the logged in user's home directory, or into the root
		directory if the user has no home directory.
	 	@param dc DbControl to use when accessing the database 
	 	@param dir Directory where the file to upload is located. 
	 	@param filename The file's name.
	 	@param fileTypeId The id of the file type in Base. Null is allowed.
	 	@return The File item that has been created for the uploaded file.
	 	@throws Exception If the upload fails.
		
	*/
	public static File uploadFile(DbControl dc, String dir, String filename, String fileTypeId)
		throws Exception
	{
		String path = dir + "/" + filename;
		TestUtil.write("--Uploading file: " + path + "\n");
		
		boolean isBzip = false;
		boolean isTar = false;
		if (filename.endsWith(".bz2"))
		{
			isBzip = true;
			filename = filename.substring(0, filename.length()-4);
		}
		if (filename.endsWith(".tar"))
		{
			isTar = true;
			filename = filename.substring(0, filename.length() - 4);
		}
	
		Path pathOnBase = new Path("~/" + filename, Path.Type.FILE);
		Directory dirOnBase = Directory.getNew(dc, pathOnBase);
		File file = File.getFile(dc, dirOnBase, pathOnBase.getFilename(), true);
		if (fileTypeId != null)
		{
			file.setItemSubtype(ItemSubtype.getById(dc, SystemItems.getId(fileTypeId)));
		}
		if (!file.isInDatabase()) 
		{
			dc.saveItem(file);
			InputStream toUpload = null;
			try
			{
				toUpload = TestUtil.class.getResourceAsStream(path);
				if (toUpload == null)
				{
					TestUtil.write("--NOT FOUND - skipping upload\n");
				}
				else
				{
					if (isBzip) 
					{
						// Remove 'BZ' marker at start of file
						toUpload.read();
						toUpload.read();
						toUpload = new CBZip2InputStream(toUpload);
					}
					if (isTar)
					{
						// Assume that the TAR only contains a single file
						TarInputStream tar = new TarInputStream(toUpload);
						tar.getNextEntry();
						toUpload = tar;
					}
					file.upload(toUpload, false, true);
				}
			}
			finally
			{
				net.sf.basedb.util.FileUtil.close(toUpload);
			}
		}
		return file;
	}

	/**
		Add a file to the file set of a FileStoreEnabled item. Optionally,
		validate and extract metadata.
	*/
	public static FileSetMember addDataFile(DbControl dc, FileStoreEnabled item, 
		String dataFileType, File file, boolean validate)
	{
		TestUtil.write("--Adding file to " + ((Nameable)item).getName() + ": " + file.getName() + "\n");
		FileSetMember member = FileStoreUtil.setDataFile(dc, item, dataFileType, file);
		if (validate) item.getFileSet().validate(dc);
		return member;
	}
	
}
