/**
	$Id: LowessTest.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.test.performance;


import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.FloatParameterType;
import net.sf.basedb.core.IntegerParameterType;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ItemParameterType;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.test.PluginUtil;
import net.sf.basedb.test.TestUtil;
import net.sf.basedb.util.Values;

/**
	Create a normalized bioassayset with LOWESS.

	@author nicklas
	@version 2.0
	@base.modified $Date: 2011-10-26 12:51:52 +0200 (on, 26 okt 2011) $
*/
public class LowessTest
{

	public static boolean ok = true;

	public static final String DEFAULT_SOURCE_PREFIX = "Filtered";
	public static final String DEFAULT_CHILD_PREFIX = "Lowess";

	
	/**
		Run the tests in this class.
	*/
	public static boolean run(String user, String password,
		String sourcePrefix, String childPrefix, boolean execute)
	{
		DbControl dc = null;
		TestUtil.write("++Entering LOWESS test\n");
		try
		{
			TestUtil.login(user, password);

			// Configuration options
			Float fitFraction = Values.getFloat(Config.get("lowess.fitFraction"), 0.33f);
			Float delta = Values.getFloat(Config.get("lowess.delta"), 0.1f);
			Integer iterations = Values.getInteger(Config.get("lowess.iterations"), 4);
			Integer blockGroup = Values.getInteger(Config.get("lowess.blockGroup"), 0);
			if (sourcePrefix == null) sourcePrefix = DEFAULT_SOURCE_PREFIX;
			if (childPrefix == null) childPrefix = DEFAULT_CHILD_PREFIX;
			
			TestUtil.write("---------------------------------------------\n");
			TestUtil.write(" Fit fraction=" + fitFraction + "\n");
			TestUtil.write("        Delta=" + delta + "\n");
			TestUtil.write("   Iterations=" + iterations + "\n");
			TestUtil.write("  Block group=" + blockGroup + "\n");
			TestUtil.write("Source prefix=" + sourcePrefix + "\n");
			TestUtil.write(" Child prefix=" + childPrefix + "\n");
			TestUtil.write("      Execute=" + execute + "\n");
			TestUtil.write("---------------------------------------------\n");

			// Load source bioassay sets
			dc = TestUtil.getDbControl();
			Experiment experiment = PrepareTest.getExperiment(dc);
			List<BioAssaySet> sources = new ArrayList<BioAssaySet>(
					Util.findAllBioAssaySets(dc, experiment, sourcePrefix + "%"));
			int numSources = sources.size();
			if (numSources == 0)
			{
				throw new ItemNotFoundException("BioAssaySet[name like " + sourcePrefix + "]");
			}
			dc.close();

			// Run Lowess tests
			long minTime = Long.MAX_VALUE;
			long maxTime = 0;
			long totalTime = 0;
			for (int i = 0; i < numSources; ++i)
			{
				BioAssaySet source = sources.get(i);
				String lowessName = source.getName().replace(sourcePrefix, childPrefix);
				long time = createNormalizedBioAssaySet(source, lowessName, 
						fitFraction, delta, iterations, blockGroup, execute) / 1000;
				if (execute)
				{
					totalTime += time;
					if (time < minTime) minTime = time;
					if (time > maxTime) maxTime = time;
					TestUtil.write(lowessName + "\t" + Values.formatTime(time) + "\n");
				}
				else
				{
					TestUtil.write("--Lowess job created for: " + source.getName() + "-->" + 
							lowessName + "\n");
				}
			}

			if (execute)
			{
				TestUtil.write("---------------------------------------------\n");
				TestUtil.write("Max time: " + Values.formatTime(maxTime) + "\n");
				TestUtil.write("Min time: " + Values.formatTime(minTime) + "\n");
				TestUtil.write("Mean time: " + Values.formatTime(totalTime / numSources) + "\n");
			}
			TestUtil.write("---------------------------------------------\n");
			TestUtil.logout();
		}
		catch (Throwable t)
		{
			ok = false;
			t.printStackTrace();
		}
		finally
		{
			if (dc != null) dc.close();
		}
		TestUtil.write("++Leaving LOWESS test (" + (ok ? "OK" : "FAILED") + ")\n");
		return ok;
	}

	/**
		Create a filtered bioassay set.
	*/
	public static long createNormalizedBioAssaySet(BioAssaySet bioAssaySet, String name, 
		Float fitFraction, Float delta, Integer iterations, Integer blockGroup, boolean execute)
		throws Exception
	{
		if (execute) TestUtil.write("--Create LOWESS bioassay set: " + name + "\n");
		DbControl dc = TestUtil.getDbControl();
		try
		{		
			PluginDefinition plugin = PluginDefinition.getByClassName(dc, "net.sf.basedb.plugins.LowessNormalization");
			bioAssaySet = BioAssaySet.getById(dc, bioAssaySet.getId());
			Job job = Job.getNew(dc, plugin, null, bioAssaySet.getExperiment());
			job.setName("PerformanceTest->Lowess: " + name);
			job.setSendMessage(false);
			job.setRemoveJobWhenFinished(true);
			job.setParameterValue("source", 
					new ItemParameterType<BioAssaySet>(BioAssaySet.class, null), bioAssaySet);
			job.setParameterValue("childName", 
					new StringParameterType(255, null, true), name);
			job.setParameterValue("fitFraction", new FloatParameterType(), fitFraction);
			job.setParameterValue("delta", new FloatParameterType(), delta);
			job.setParameterValue("iter", new IntegerParameterType(), iterations);
			job.setParameterValue("blockGroup", new IntegerParameterType(), blockGroup);
			dc.saveItem(job);
			dc.commit();
			long time = 0;
			if (execute)
			{
				time = System.currentTimeMillis();
				PluginUtil.executeJob(job);
				time = System.currentTimeMillis() - time;
			}
			return  time;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	
}
